module OEHClient

	module Exception

		# Used when an invalid OAuth consumer is detected as part of the client-side API calls
		class InvalidConsumerException < ::Exception

		end

		# InvalidSpaceConfigException is used when a OEHClient::Space instance is missing one or more
		#  of the key data attributes
		class InvalidSpaceConfigException < ::Exception

		end

		# InvalidSpaceException is used when a space is not configured
		class InvalidSpaceException < ::Exception

		end

		# InvalideSpaceObjectException is used when a method expects a OEHClient::Config::Space object and
		#  the object passed is not of the same type
		class InvalidSpaceObjectException < ::Exception

		end

		# HTTPRequestException is used to manage any REST error code that is returned by the OEH
		#  server
		class HTTPRequestException < ::Exception

			attr_accessor :http_code

			# override the constructor to add the code value to exception class
			def initialize(msg, code)
				super(msg)
				@http_code = code
			end
			
		end

		# Missing ParameterException is used to identify the list of parameters that are required, but missing
		#  by a key method 
		class MissingParameterException < ::Exception

			def initialize(parameter_list)

				super("Missing the required parameters #{parameter_list.join(', ')}")
			end
			
		end

		class IncorrectParameterTypeException < ::Exception


			def initialize(expected_kind, passed_kind)

				super("Incorrect Parameter Type.  Expected #{expected_kind} but received #{passed_kind}")
			end
			
		end

	end

end